#!/bin/bash
out_dir=plots
in_dir=output_test
mode=${1:-"x11"}
plot_id=1



#plot of throughput
function plot_thr()
{
    in_file_name=$1
    out_file_name=$2
		title=$3

case $mode in 
    eps)
      printf "  set terminal post eps color font \"arial,20\" linewidth 2
			  				set output '$out_file_name.eps'
             " > tmp.txt
			options="-mono"
		  ;;
		png)
			printf "  set terminal png size 700, 350 font \"arial,14\"
                set output '$out_file_name.png'
						 " > tmp.txt
			options="-persist"
			;;
		*)
			printf "  set terminal x11 size 700, 350 font \"arial,14\"

						 " > tmp.txt
			options="-persist"
esac

    printf "
set termoption enhanced
set encoding utf8
set style data linespoints
set grid nopolar
set grid noxtics noztics nomztics nox2tics nomx2tics noy2tics nomy2tics nocbtics nomcbtics
set grid layerdefault   linetype 0 linewidth 1.000, linetype 0 linewidth 1.000
set key inside right bottom vertical Right noreverse enhanced autotitles nobox samplen 1
set pointsize 1.5
set xtics scale 1.0, 1.0
set xtics (1,40,80,120,160,200,240,280,320,360,400)
set mytics
set ytics scale 2.0, 1.0
#set xrange [ 1 : 300 ] noreverse nowriteback
set ylabel \"Normalized throughput\" 
set xlabel \"MAC-SAL buffer size Q [pkts]\" 
set yrange [ 0 : 1 ] noreverse nowriteback
set bmargin  3.1
plot '$in_file_name' using 1:2 index 1 with lp lw 2 title 			columnhead(1),      ''           using 1:2 index 0 with  	lp 	lw 2	title   columnhead(1),      ''           using 1:2 index 2 with 		lp	lw 2 pt 11 title   columnhead(1)    	" >> tmp.txt


    echo $plot_curves >> tmp.txt
    
    gnuplot $options < tmp.txt

}


#plot of Time Worst Case Fair Index interm
function plot_twfi_interm()
{
    in_file_name=$1
    out_file_name=$2
		title=$3

case $mode in 
    eps)
      printf "  set terminal post eps color 22 font \"arial,20\" linewidth 2
			  				set output '$out_file_name.eps'
             " > tmp.txt
			options="-mono"
		  ;;
		png)
			printf "  set terminal png size 700, 350 font \"arial,14\"
                set output '$out_file_name.png'
						 " > tmp.txt
			options="-persist"
			;;
		*)
			printf "  set terminal x11 size 700, 350 font \"arial,14\"

						 " > tmp.txt
			options="-persist"
esac    

    printf "
set termoption enhanced
set encoding utf8
set style data linespoints
set grid nopolar
set grid noxtics noztics nomztics nox2tics nomx2tics noy2tics nomy2tics nocbtics nomcbtics
set grid layerdefault   linetype 0 linewidth 1.000, linetype 0 linewidth 1.000
set key inside left center vertical Right noreverse enhanced autotitles nobox samplen 1
set pointsize 1.5
set xtics scale 1.0, 1.0
set xtics (1,40,80,120,160,200,240,280,320,360,400)
set mytics
set ytics scale 2.0, 1.0
#set xrange [ 1 : 400 ] noreverse nowriteback
set ylabel \" T-WFI [ms] \" 
set xlabel \"MAC-SAL buffer size Q [pkts]\" 
#set yrange [ 0 : 2 ] noreverse nowriteback
set bmargin  3 
plot '$in_file_name' using 1:6 index 0 with lp lw 2 title 			columnhead(1),      ''           using 1:6 index 3 with 		lp	lw 2 title   columnhead(1),      ''           using 1:6 index 2 with 		lp lw 2 pt 11	 title   columnhead(1)
" >> tmp.txt


    echo $plot_curves >> tmp.txt
    
    gnuplot $options < tmp.txt

}



#plot of execution time of the tool
function plot_exec_time()
{
    in_file_name=$1
    out_file_name=$2
		title=$3

case $mode in 
    eps)
      printf "  set terminal post eps color 22 font \"arial,20\" linewidth 2
			  				set output '$out_file_name.eps'
             " > tmp.txt
			options="-mono"
		  ;;
		png)
			printf "  set terminal png size 700, 350 font \"arial,14\"
                set output '$out_file_name.png'
						 " > tmp.txt
			options="-persist"
			;;
		*)
			printf "  set terminal x11 size 700, 350 font \"arial,14\"

						 " > tmp.txt
			options="-persist"
esac    

    printf "
set termoption enhanced
set encoding utf8
set style data linespoints
set grid nopolar
set grid noxtics   nomztics nomx2tics noy2tics nomy2tics nocbtics nomcbtics
set grid layerdefault   linetype 0 linewidth 1.000, linetype 0 linewidth 1.000
set key inside left  top  vertical Right noreverse enhanced autotitles nobox samplen 1
set pointsize 1.5
set xtics scale 1.0, 1.0
set xtics (1,40,80,120,160,200,240,280,320,360,400)
set mytics
set ytics scale 2.0, 1.0
#set xrange [ 1 : 400 ] noreverse nowriteback
set ylabel \"Total execution time [ns] \"
set xlabel \"MAC-SAL buffer size Q [pkts]\" 
set yrange [ 0 : 500 ] noreverse nowriteback
set bmargin  3 
plot '$in_file_name' using 1:7 index 0 with lp lw 2 title 			columnhead(1),      ''           using 1:7 index 1 with  	lp	lw 2	title   columnhead(1)    	" >> tmp.txt

    echo $plot_curves >> tmp.txt
    
    gnuplot $options < tmp.txt

}

plot_thr $in_dir/new_thr.out $out_dir/thr_analogical analogical
plot_twfi_interm $in_dir/total_test_analogical.out $out_dir/total_twfi_interm_analogical analogical
plot_exec_time $in_dir/total_test_analogical.out $out_dir/total_exec_time_analogical analogical


rm tmp.txt


