/*
 * $Id$
 *
 * userspace compatibility code for dummynet schedulers
 */

#ifndef _DN_TEST_H
#define _DN_TEST_H
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>	/* bzero, ffs, ... */
#include <string.h>	/* strcmp */
#include <errno.h>
#include <math.h>
#include <inttypes.h>
#include <sys/queue.h>
#include <sys/time.h>


extern int debug;
#define ND(fmt, args...) do {} while (0)
#define D1(fmt, args...) do {} while (0)
#define D(fmt, args...) fprintf(stderr, "%-8s " fmt "\n",      \
        __FUNCTION__, ## args)
#define DX(lev, fmt, args...) do {              \
        if (debug > lev) D(fmt, ## args); } while (0)


#define offsetof(t,m) (intptr_t)((&((t *)0L)->m))

#include <mylist.h>

/* prevent include of other system headers */
#define	_NETINET_IP_VAR_H_	/* ip_fw_args */
#define _IPFW2_H
#define _SYS_MBUF_H_

enum	{
	DN_QUEUE,
};

enum	{
	DN_SCHED_FIFO,
	DN_SCHED_WF2QP,
};

struct dn_id {
	int type, subtype, len, id;
};
struct dn_fs {
	int par[4];	/* flowset parameters */


	char * const *args;
	int n_args;
	int qos_flows;

	/* simulation entries.
	 * 'index' is not strictly necessary
	 * y is used for the inverse mapping ,
	 */
	int index;
	int y;	/* inverse mapping */
	int base_y;	/* inverse mapping */
	int next_y;	/* inverse mapping */
	int n_flows;
	int first_flow;
	int next_flow;	/* first_flow + n_flows */
	/*
	 * when generating, let 'cur' go from 0 to n_flows-1,
	 * then point to flow first_flow + cur
	 */
	int	cur;
};
struct dn_sch {
};
struct dn_flow {
	struct dn_id oid;
	int length;               /* length of the flow [pkts] */
	int max_length;
	int total_length;         /* TOTAL flow length, 
                               * merge the QoS flow and the MAC flow associeted 
                               */
	int len_bytes;            /* length of the flow [Byte] */
	int drops;                /* number of packets dropped */
	uint64_t tot_bytes;
	uint32_t flow_id;
	struct list_head h;    	  /* used by the generator */
	uint32_t pkts;			  /* number of packets served by the flow */
	uint32_t wind_pkts;       /* number of packets served by the flow in
                               * a particular window time.
                               * Used for stats in dynamic scenario.
                               */
	uint32_t loss_pkts;        /* number of packets lost */
	uint32_t bklgd_time;	   /* backlog time of the flow [pkts] */
	uint32_t bklgd_start_time; /* start time [pkts] of backlog period */
	uint32_t pkt_serv_bklgd;   /* packet served by the flow in the
                                * backlogged period 
                                */
	uint32_t pkts_all_blog;   /* packet served by the flow in the 
                               * COMMON backlog period, calculate as
                               * backlog time intersection between all
                               * flows
                               */
	uint32_t allpkt_serv_bklgd; /* total number of packets served by
                                 * all flows during the backlog period
                                 * of this specific flow
                                 */
	int backlog;   /* used to track the backlog/unbacklog status of flow */
	uint32_t q_time_min;     /* min queueing delay experience [pkts] of a packet
	                          * for this flow
	                          */
	uint32_t q_time_max;     /* max queueing delay experience [pkts] of a packet
	                          * for this flow
	                          */
	double q_avg_time;   /* average queueing delay experience [pkts] of a packet
	                      * for this flow
	                      */
	double q_time_square; /* stdr deviation of queueing delay [pkts] of a packet
	                       * for this flow
	                       */
	int ploss;      /* class of packet loss probability for this flow
	                 * it's an index of a static array containing differents
	                 * possible static packet loss probability
	                 */
};
struct dn_link {
};

struct ip_fw_args {
};

struct mbuf {
        struct {
                int len;
        } m_pkthdr;
    struct mbuf *m_nextpkt;

    /* both start/end time are an integer number of dequeue event */
    uint32_t enq_time;    /* enqueue time [pkts] of a packet in the scheduler */
    uint32_t enq_mac_time; /* enqueue time [pkts] of a packet in 
                            * it's specific MAC flow 
                            */

    uint32_t Q_bklg; /* queue lenght [pkts] just after
                      * the enqueue of this packet 
                      */


	uint32_t flow_id;	  /* index of flow */
	uint32_t father_flow_id; /* used by pakets into the MAC_SAL
                              * it contains the flow_id of the corresponding 
                              * QoS (global) flow
                              */
	void *cfg;	/* config args */
	/*
	 * 8 * 208 byte padding, to get a somewhat realistic buffer size
	 * (http://comments.gmane.org/gmane.linux.network/229464)
	 */
	uint64_t pad[208];
};

#define MALLOC_DECLARE(x)
#define KASSERT(x, y)	do { if (!(x)) printf y ; exit(0); } while (0)
struct ipfw_flow_id {
};

typedef void * module_t;
struct _md_t {
	const char *name;
	int (*f)(module_t, int, void *);
	void *p;
};
typedef struct _md_t moduledata_t;
#define DECLARE_MODULE(name, b, c, d)	\
	moduledata_t *_g_##name = & b
#define MODULE_DEPEND(a, b, c, d, e)

#ifdef IPFW
#include <ipfw/dn_heap.h>
#include <ipfw/ip_dn_private.h>
#include <ipfw/dn_sched.h>
#else
struct dn_queue {
        struct dn_fsk *fs;             /* parent flowset. */
        struct dn_sch_inst *_si;	/* parent sched instance. */
};
struct dn_schk {
};
struct dn_fsk {
	struct dn_fs fs;
	struct dn_schk *sched;
};
struct dn_sch_inst {
	struct dn_schk *sched;
};
struct dn_alg {
	int type;
	const char *name;
	void *enqueue, *dequeue;
	int q_datalen, si_datalen, schk_datalen;
	int (*config)(struct dn_schk *);
	int (*new_sched)(struct dn_sch_inst *);
	int (*new_fsk)(struct dn_fsk *);
    	int (*new_queue)(struct dn_queue *q);
    	int (*update_queue)(struct dn_queue *q);
};

#endif

static inline void
mq_append(struct mq *q, struct mbuf *m)
{
        if (q->head == NULL)
                q->head = m;
        else
                q->tail->m_nextpkt = m;
        q->tail = m;
        m->m_nextpkt = NULL;
}

#endif /* _DN_TEST_H */
